# -*- coding: utf-8 -*-
from os import path
from StringIO import StringIO
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from OFS.Folder import Folder
from OFS.PropertyManager import PropertyManager
from OFS.FindSupport import FindSupport
from Products.CMFCore.utils import UniqueObject, getToolByName
from Globals import InitializeClass
from Settings import ApplicationToolName, ApplicationToolMetaType, DatabaseToolName
from Permissions import RakunView, ManagePortal
from sql import Condition
from Settings import EntityMetaType, InterfaceMetaType, RelationToolName
from RakunBaseAppTool import RakunBaseAppTool

class ApplicationTool(Folder, UniqueObject, RakunBaseAppTool):
    """
       Application Tool 
    """
    toolicon = 'zmi/application-tool.gif'
    id = ApplicationToolName
    meta_type = ApplicationToolMetaType
    
    relations = []
    db_versions = {}
    db_migration_script = "migrate_database"
    
    _properties = ({'id':'title', 'type':'string', 'mode':'w'},
                   {'id':'db_migration_script', 'type':'string', 'mode':'w'},)

    manage_options = (Folder.manage_options +\
                      ({'label':'Export',
                        'action':'exportSettings'},                       
                       {'label':'Database Settings',
                        'action':'databaseSettings'},))
    
    security = ClassSecurityInfo()       
    exportSettings = PageTemplateFile('zmi/exportSettings', globals())
    databaseSettings = PageTemplateFile('zmi/databaseSettings', globals())
  
    def index_html(self,REQUEST=None):
        """ """
        return self.rakun_application_view(REQUEST)
    
    def manage_migrateRelations(self,REQUEST=None):
        """all relations move to rakun relation tool """
        relation_tool = getToolByName(self, RelationToolName)
        for relation in self.relations:
            relation_tool.addRelation(relation['master_entity'], 
                                      relation['master_fields'].split(','),
                                      relation['detail_entity'],
                                      relation['foreign_field'],
                                      relation['relation'])
        if REQUEST:
            message = 'All relations are migrated.'
            return self.relationSettings(self,REQUEST,
                                         manage_tabs_message=message)
            
            
            
    security.declareProtected(RakunView, 'relation_search')
    def relation_search(self,master_entity=None,detail_entity=None,
                        foreign_field=None):
        """ """
        relation_tool = getToolByName(self, RelationToolName)
        return relation_tool.relation_search(master_entity,detail_entity,
                                             foreign_field)
    
    security.declareProtected(RakunView,'getRelation')
    def getRelation(self,foreign):
        """ """
        relation_tool = getToolByName(self, RelationToolName)
        return relation_tool.getRelation(foreign)
            
    security.declareProtected(RakunView,'getMasterFields')
    def getMasterFields(self, foreign):
        """ """
        relation_tool = getToolByName(self, RelationToolName)
        return relation_tool.getMasterFields(foreign)
    
    security.declareProtected(RakunView,'getMasterEntity')    
    def getMasterEntity(self, foreign):
        """ """
        relation = self.getRelation(foreign)
        if relation:
            master_entity = self.restrictedTraverse(\
                relation[0]['master_entity'])
            return master_entity
        
    def getEntityObject(self, entity_path):
        return self.unrestrictedTraverse(entity_path)
    
    security.declareProtected(RakunView,'getJoinCondition')
    def getJoinCondition(self,foreign):
        """ """
        master_entity = self.getMasterEntity(foreign)
        if master_entity:
            primary_key = master_entity.getPrimaryField()
            return Condition(primary_key,'=',foreign)
        
    security.declareProtected(RakunView,'getAllEntities')
    def getAllEntities(self):
        return FindSupport.ZopeFind(FindSupport(self), self, [],
                                    [EntityMetaType],search_sub=1)
    
    security.declareProtected(RakunView,'getAllDetailEntities')
    def getAllDetailEntities(self,master_entity):
        """ """
        relations = self.relation_search(master_entity=master_entity)
        return map(lambda r: r['detail_entity'], relations)
    
    security.declareProtected(RakunView,'getAllInterface')
    def getAllInterface(self,master_entities=[]):
        """ """
        interfaces = FindSupport.ZopeFind(FindSupport(self), self, [],
                                          [InterfaceMetaType],search_sub=1)
        if master_entities:
            return filter(lambda interface:interface[1].master_entity in\
                          master_entities, interfaces)
        else:
            return interfaces
    
    security.declareProtected(ManagePortal, 'manage_export')
    def manage_export(self,export_path,REQUEST=None):
        """rakun applications export method"""
        message = StringIO()
        message.write("All modules are exported to %s\n"%export_path)
        for id in self.objectIds():             
            ob=self._getOb(id)
            if ob.meta_type != 'Filesystem Directory View':
                f = path.join(export_path, '%s.%s' % (id, 'zexp'))
                ob._p_jar.exportFile(ob._p_oid, f)
                message.write("%s \n"%(f))
            
        if REQUEST:            
            return self.exportSettings(self,REQUEST,
                                       manage_tabs_message=message.getvalue())
        
     
    
    #---------------------------------------------------------------------------
    
    security.declareProtected(ManagePortal, 'manage_getDatabaseInfo')
    def manage_getDatabaseInfo(self):
        """ """
        db_info = []
        connectionids = []
        for path,entity in self.getAllEntities():
            connectionid = entity.getConnectionId()
            if not connectionid in connectionids:
                dbobj = entity.getDBObject()
                db_info.append({'dbfamily' : entity.getDBFamily(),
                                'connectionid' : connectionid,
                                'version_no' : dbobj.getVersion()})
                connectionids.append(connectionid)
        return db_info
    
    security.declareProtected(ManagePortal, 'manage_setDatabaseVersion')
    def manage_setDatabaseVersion(self,databases=[],REQUEST=None):
        """ """
        self.db_versions = {}
        for database in databases:
            self.db_versions[database['connectionid']] = database['version_no']
        self.db_versions = self.db_versions
        if REQUEST:
            message="Application Tool Database Versions Updated"
            return self.databaseSettings(self,REQUEST,
                                         manage_tabs_message=message)  
    
    security.declareProtected(ManagePortal, 'manage_migrate')
    def manage_migrate(self):
        """ """
        dbtool = getToolByName(self, DatabaseToolName)
        dbobj = dbtool.getDBObject(dbfamily,connectionid)
        dbobj.setVersion(version_no)
  
    # Veritaban�rt/export'unda sequence do�iletilemedi��
    security.declareProtected(ManagePortal, 'sync_sequence')
    def sync_sequences(self,REQUEST=None):
        """ sycronize database sequences
        """
        for entityid,entity in self.getAllEntities():
            entity.sync_sequence()
        if REQUEST:
            message="All sequences are syncronized."
            return self.databaseSettings(self,REQUEST,
                                         manage_tabs_message=message) 
        
    #---------------------------------------------------------------------------
InitializeClass(ApplicationTool)    


