#from AccessControl import allow_class
#from Interface.Implements import implements
from IObjects import ISql
from Settings import registerRakunClass


class BaseCondition:    
    
    operator = ''
    
    def __init__(self, field, value):
        self.field = field
        self.value = value
        
    def __and__(self, other):
        return And(self,other)
    
    def __or__(self, other):
        return Or(self,other)
    
    def __invert__(self):
        return Not(self)
    
    def __eq__(self, other):
        return isinstance(other, BaseCondition) and (self.field == other.field)\
               and (self.operator == other.operator) and (self.value == other.value)
        
        
    def render(self, dbobject):
        if ISql.isImplementedBy(self.value):
            value = self.value.render_sql_condition()
        else:
            value = dbobject.setEscapeChars(self.value)
            value = self.field.render_sql_value(dbobject, value)
            
        return '(%s %s %s)' %(self.field.render_sql_condition(),
                              self.operator, 
                              value )


class Eq(BaseCondition):    
    
    operator = '='

registerRakunClass('Eq', Eq)

class Ge(BaseCondition):
    
    operator = '>='

registerRakunClass('Ge', Ge)

class Gt(BaseCondition):    
    
    operator = '>'

registerRakunClass('Gt',Gt)    


class Le(BaseCondition):    

    operator = '<='

registerRakunClass('Le', Le)


class Lt(BaseCondition):
    
    operator = '<'

registerRakunClass('Lt', Lt)


class Ne(BaseCondition):

    operator = '<>'
    
registerRakunClass('Ne', Ne)


class MatchGlobe(BaseCondition):    
    
    def render(self, dbobject):
        dbtype = self.field.get_db_type()
        if dbtype in ['string','text', 'date']:
            value = '%' + self.value + '%'
            # (fieldname like value)
            return "(%s %s '%s')" %(self.field.render_sql_condition(), 
                                       dbobject.like_keyword,
                                       dbobject.setEscapeChars(value) )
        else:
            newCondition = Eq(self.field, self.value)
            return newCondition.render(dbobject)

registerRakunClass('MatchGlobe', MatchGlobe)


class In(BaseCondition):

    pass

registerRakunClass('In', In)


class Between(BaseCondition):

    pass

registerRakunClass('Between', Between)


class Not(BaseCondition):
    """ ~(query) """
    
    def __init__(self, query):
        self._query = query
    
    def render(self):
        pass
    
registerRakunClass('Not', Not)


class CompositeCondition(BaseCondition):
    
    operator = ''
    
    def __init__(self, *conditions):
        self._conditions= []
        for c in conditions: self.addCondition(c)

    def __eq__(self,other):
        return isinstance(other, CompositeCondition) and self._conditions == other._conditions
        
        
        
    def addCondition(self, condition):
        assert isinstance(condition, BaseCondition)
        if condition not in self._conditions:
            self._conditions.append(condition)
    
   
    def render(self, dbobject):
        conditions = [condition.render(dbobject) for condition in self._conditions]
        return ' (%s) ' %self.operator.join(conditions)
        

class And(CompositeCondition):
    operator = ' AND '

registerRakunClass('And', And)


class Or(CompositeCondition):
    
    operator = ' OR '

registerRakunClass('Or', Or)