# -*- coding: utf-8 -*-
from Products.Formulator.Form import ZMIForm as BaseForm
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo, getSecurityManager
from Globals import DTMLFile, InitializeClass
from Products.Formulator.Form import create_settings_form,fields
from Permissions import permissions, RakunAccessDatabase, RakunView
from EntityProperties import EntityProperties
from DataManagement import DataManager
from Module import Module
from Settings import EntityMetaType, ApplicationToolName, i18ndomain
from RakunUtils import translate
from IObjects import IRpc

manage_addEntityForm = PageTemplateFile('zmi/addEntity', globals(), 
                                        __name__="manage_addEntityForm")

additional_meta_types = [{'action': 'manage_addProduct/OFSP/fileAdd', 'name': 'File'},
                         {'action': 'manage_addProduct/OFSP/methodAdd', 'name': 'DTML Method'},
                         {'action': 'manage_addProduct/PythonScripts/pyScriptAdd', 'name': 'Script (Python)'}]

default_text = '''
<tal:block define="absolute_url template/absolute_url;
                   entity       nocall:template;">
    <metal:block use-macro="here/entity_template/macros/master" />
</tal:block>
'''

class Entity(Module,BaseForm,EntityProperties,DataManager,ZopePageTemplate):
    """ 
        Rakun Entity
    """
    
    meta_type = EntityMetaType
    
    om_icons = None
    
    _text = default_text
    __implements__ = (IRpc,)
    
    manage_options = (BaseForm.manage_options +
                      EntityProperties.manage_options +
                      ({'label':'Edit', 'action':'pt_editForm'},
                       {'label':'View', 'action':'index_html'},)
                      )
    
    _properties = EntityProperties._properties
    security = ClassSecurityInfo()
    
    #---------------------------------------------------------------------------
    def getProperties(self):
        properties = {}
        properties['primaryid'] = self.getPrimaryKey()
        properties['url'] = self.absolute_url()
        properties['id'] = self.getId()
        properties['title'] = self.get_i18n_message( self.getTitle() ) 
        properties['__class__'] = 'Entity'
        properties['path'] = '/'.join(self.getPhysicalPath())
        return properties
        
    def dump(self, marshaller, write):
        properties = self.getProperties()
        properties['fields'] = self.get_fields()
        marshaller.dump_struct(properties, write)
        
    
    def load(self):
        pass
    
    
    def getObject(self):
        """ """
        return self
    
    #---------------------------------------------------------------------------    
    
    def all_meta_types(self):
        meta_types = [ ftype for ftype in self._meta_types if str(ftype.get("name","")).find("Rakun")>=0 ]
        return meta_types + additional_meta_types
    
    
    security.declareProtected(RakunView,"get_fieldids_in_group")
    def get_fieldids_in_group(self, group, include_disabled=0):
        """ """
        return [field.getId() for field in self.get_fields_in_group(group,
                                                                    include_disabled)]
    
    
    security.declareProtected(RakunView,"getApplicationPath")
    def getApplicationPath(self):
        url_tool = getToolByName(self, 'portal_url')
        relativeUrl = url_tool.getRelativeUrl
        application_tool = getToolByName(self, ApplicationToolName)
        entity_path = url_tool.getRelativeContentPath(self)
        if relativeUrl(self).startswith(relativeUrl(application_tool)):
            application_path = url_tool.getRelativeContentPath(application_tool)
            path = '/'.join(entity_path[len(application_path):])
        else:
            portal_path = str(url_tool)
            path = '/'.join(entity_path[len(url_tool):])
        return path
    
    security.declareProtected(RakunView,"getInterfaceStyles")
    def getStyleObject(self):
        """ Returns entity style object
        """
        return self._getObject("custom.css")
        
    security.declareProtected(RakunView,"getInterfaceJScripts")
    def getJScriptObject(self):
        """ Returns interface jscript file objects
        """
        return self._getObject("custom.js")
    
    security.declareProtected(RakunView,"getPortalPath")
    def getPortalPath(self):
        url_tool = getToolByName(self, 'portal_url')
        relativeUrl = url_tool.getRelativeUrl
        return '/'.join(url_tool.getRelativeContentPath(self))
    
    security.declareProtected(RakunView,"getFieldRelationCondition")
    def rpcFieldRelationCondition(self,fieldid,REQUEST=None):
        """ Returns field relation-condition for foreign records
        """
        field = self.get_field(fieldid)
        return field.rpcRelationCondition(REQUEST)
    
    security.declareProtected(RakunView,"getRenderAttributes")
    def getRenderAttributes(self,fieldids,REQUEST={},get_all_attributes=1,
                            foreign=None):
        """ fieldids = [(fieldid,inputid),] foreign=field_portal_path
        """
        self._setContext(**REQUEST)
        if foreign: 
            self.set_variable('foreign',self.restrictedTraverse(foreign))
        attrs = {}
        
        if get_all_attributes:
            attrs['form_title'] = self.get_i18n_message( self.title_or_id() )
            attrs['group_rows'] = self.get_group_rows()
            attrs['group_fields'] = {}
            attrs['group_names'] = {}
            for groups in attrs['group_rows']:
                for group in groups:
                    attrs['group_fields'][group] = self.get_fieldids_in_group(group)
                    attrs['group_names'][group] = self.get_i18n_message(group)

        attrs['app_path'] = self.getApplicationPath()
        attrs['portal_path'] = self.getPortalPath()
 
        attrs['fields'] = {}       
        for fieldid,inputid in fieldids:
            field = self.get_field(fieldid)
            if field:
                value =  REQUEST.get(inputid,None)
                attrs['fields'][fieldid] = \
                     {'title'       : str(field.get_value('title')),
                      'required'    : field.get_value('required'),
                      'hidden'      : field.get_value('hidden'),
                      'render'      : field.render(value),
                      'value'       : value,
                      'icon'        : field.icon,
                      'app_path'    : '%s/%s' % (attrs['app_path'],fieldid),
                      'portal_path' : '%s/%s' % (attrs['portal_path'],fieldid)}
        return attrs
    
    security.declareProtected(RakunView,"getAttributes")
    def getAttributes(self,fieldids=[],foreign=None,REQUEST=None):
        """ """
        if REQUEST:
            self._setContext(**REQUEST)
            
        result = self.getProperties() 
        result['fields'] = []
        app_tool = getToolByName(self,ApplicationToolName)
        if not fieldids:
            fieldids = self.get_field_ids()
        for fieldid in fieldids:
            field = self.get_field(fieldid)
            if field:
                attrs = field.getProperties()
                if foreign:
                    attrs['inputname'] = foreign.getId() +'_'+ attrs['id']
                
                master_fields = app_tool.getMasterFields(field)
                if master_fields:
                    master_entity = master_fields[0].getEntity()
                    master_fieldids = [mfield.getId() for mfield in master_fields]
                    attrs['reference'] = master_entity.getAttributes(master_fieldids,
                                                                     foreign=field,
                                                                     REQUEST=REQUEST)
                result['fields'].append(attrs)
        return result
    
    security.declareProtected(RakunView,"getPermissions")
    def getPermissions(self):
        """return user permissions"""
        sm = getSecurityManager()
        user_permissions = {}
        for perm in permissions:
            if sm.checkPermission(perm, self):
                user_permissions[perm] = True
            else:
                user_permissions[perm] = False
        return user_permissions
        
        
InitializeClass(Entity)


def manage_addEntity(self,id,title='',REQUEST=None):
    " "
    entity = Entity(id,title)
    self._setObject(id,entity)
    entity._text = default_text
    if REQUEST is not None:
        url = self.DestinationURL() + '/' + id + '/manage_main'
        REQUEST['RESPONSE'].redirect(url)

