# -*- coding: utf-8 -*-
import string
from AccessControl import ClassSecurityInfo
from Globals import DTMLFile, InitializeClass
from Products.ZSQLMethods.SQL import SQLConnectionIDs
from Products.CMFCore.utils import getToolByName
from Products.Formulator.Form import BasicForm,fields
from Products.Formulator.Field import ZMIField
from Products.Formulator.Errors import FormValidationError
from RakunUtils import translate
from Settings import ApplicationToolName, i18ndomain
from Permissions import RakunView
from sql.Sql99 import sqlClasses

def createFORMSettingsForm():
    form = BasicForm('manage_FORMSettings')
    
def createMessageSettingsForm():
    form = BasicForm('manage_MessageSettings')
    insert_message = fields.TextAreaField('insert_message',
                                          title='Insert Message')
    
    update_message = fields.TextAreaField('update_message',
                                          title='Update Message')
    
    delete_message = fields.TextAreaField('delete_message',
                                          title='Delete Message')
    
    form.add_fields([insert_message,update_message,delete_message])
    return form


class EntityProperties:   
    
    entity = ''
    connectionid = ''
    dbfamily = 'postgresql'
    primary_key = ''
    gridtoolbar = ['delete','search','export']
    pagesize = 20
    minpagesize = 10
    autorefresh = 60000
    dblClickTimeInterval = 200
    order = ''
    i18n_domain = 'rakun'
    insert_message = 'new_record_inserted'
    update_message = 'record_updated'
    delete_message = 'record_deleted'
    
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(RakunView)
    
    manage_options = ({'label':'Messages',   'action':'messageSettings'},)    
    
    _properties = ({'id':'title', 'type': 'string','mode':'w','label':'Title'},
                   {'id':'dbfamily', 'type': 'selection','mode':'w',
                    'label':'DB Family','select_variable':'getDBOptions'},
                   {'id':'connectionid', 'type': 'selection','mode':'w',
                    'label':'DB Connection Id',
                    'select_variable':'getDBConnectionIDs'},
                   {'id':'entity', 'type': 'string','mode':'w',
                    'label':'Table Name'},
                   {'id':'primary_key', 'type': 'selection','mode':'w',
                    'label':'Primary Key','select_variable':'get_field_ids'},
                   
                   {'id':'pagesize', 'type': 'int','mode':'w',
                    'label':'Max. Page Size'},
                   {'id':'minpagesize', 'type': 'int','mode':'w',
                    'label':'Min. Page Size'},
                   {'id':'autorefresh', 'type': 'int','mode':'w',
                    'label':'Auto Refresh Time Interval(ms)'},
                   {'id':'dblClickTimeInterval', 'type': 'int','mode':'w',
                    'label':'Double Click Time Interval(ms)'},
                   {'id':'order', 'type': 'selection','mode':'w',
                    'label':'Default Ordered Column',
                    'select_variable':'get_field_ids'},
                   )
    
   
    form_settings = createFORMSettingsForm()               
    
    message_settings = createMessageSettingsForm()  # __init__.py içinde realize et    
    security.declareProtected('View management screens', 'messageSettings')
    messageSettings = DTMLFile('zmi/messageSettings', globals())
    
    
    def getDBOptions(self):
        return sqlClasses.keys()
    
    def getDBConnectionIDs(self):
        return SQLConnectionIDs(self)
    
    
    
    def getMasterEntities(self):
        """return ([foreign1, foreign2, ..], [master1, master2, ..])"""
        app_tool = getToolByName(self, ApplicationToolName)
        master_entities = []
        foreign_fields = []
        for field in self.get_fields():
            if field.get_value('isforeign'):
                foreign_fields.append(field)
                master_entity = app_tool.getMasterEntity(field)
                if master_entity:
                    master_entities.append( master_entity )   
                else:
                    raise "Relation Not Found: %s" % field.absolute_url()
        return foreign_fields,master_entities
    
    def getPrimaryField(self):
        """ return primary"""
        primaryid = self.getPrimaryKey()
        return self.get_field(primaryid)
    
    def getApplicationUrl(self):
        """ """
        try: return self.rakun_applications.absolute_url()
        except: return self.portal_url()
    
    def getRelativeUrl(self):
        """ """
        application_url = self.getApplicationUrl()
        return self.absolute_url()[len(application_url)+1:]
     
    # -*- DB Properties -*-
    def getEntityName(self):
        return self.entity
    
    def getConnectionId(self):
        return self.connectionid
    
    def getDBFamily(self):
        return self.dbfamily
    
    def getPrimaryKey(self):
        """ """
        return self.primary_key
    
    # -*- GRID Properties -*-
    def getGridToolbar(self):
        return self.gridtoolbar
    
    def getMaxPageSize(self):
        """ """
        return self.pagesize
    
    def getMinPageSize(self):
        """ """
        return self.minpagesize
    
    def getAutoRefreshInterval(self):
        """ """
        return self.autorefresh
    
    def getDblClickTimeInterval(self):
        """ """
        return self.dblClickTimeInterval
    
    def getOrder(self):
        """ """
        return self.order or self.primary_key
    
    def getDefaultGridSettings(self):
        """ """
        settings = {'pagesize' : self.getMaxPageSize(),
                    'minpagesize' : self.getMinPageSize(),
                    'autoRefreshInterval' : self.getAutoRefreshInterval(),
                    'dblClickTimeInterval' : self.getDblClickTimeInterval(),
                    'order' : self.getOrder() 
                    }
        return settings 
    # -*- FORM Properties -*-
    def getFormName(self):
        """ """
        return self.name
    
    # -*- Messages Properties -*-
    def get_i18n_message(self,message):
        i18n_message =  translate(self.get_i18n_domain(),message,
                                  context=self,default=message,as_unicode=1)
        if i18n_message == message:
            i18n_message =  translate(i18ndomain, message,
                                      context=self,default=message,as_unicode=1)
        return i18n_message.encode(self.get_form_encoding())
    
    # edit methods
    def manage_GRIDSettings(self,REQUEST):
        """
        grid settings
        """
        try:
            result = self.grid_settings.validate_all(REQUEST)
        except FormValidationError, e:
            message = "Validation error(s).<br />" + string.join(
                map(lambda error: "%s: %s" % (error.field.get_value('title'),
                                              error.error_text), e.errors), "<br />")
            return self.gridSettings(self, REQUEST,
                                     manage_tabs_message=message)
        # now set the form settings
        for key, value in result.items():
            setattr(self, key, value)
        message="Settings changed."
        
        return self.gridSettings(self, REQUEST,
                                 manage_tabs_message=message)
    
    def manage_DBSettings(self,REQUEST):
        """
        db settings
        """
        try:
            result = self.db_settings.validate_all(REQUEST)
        except FormValidationError, e:
            message = "Validation error(s).<br />" + string.join(
                map(lambda error: "%s: %s" % (error.field.get_value('title'),
                                              error.error_text), e.errors), "<br />")
            return self.dbSettings(self, REQUEST,
                                     manage_tabs_message=message)
        # now set the form settings
        for key, value in result.items():
            setattr(self, key, value)
        message="Settings changed."
        
        return self.dbSettings(self, REQUEST,
                               manage_tabs_message=message)
    
    def manage_FORMSettings(self,REQUEST):
        pass
    
    def manage_MessageSettings(self,REQUEST):
        """
        message settings
        """
        try:
            result = self.message_settings.validate_all(REQUEST)
        except FormValidationError, e:
            message = "Validation error(s).<br />" + string.join(
                map(lambda error: "%s: %s" % (error.field.get_value('title'),
                                              error.error_text), e.errors), "<br />")
            return self.messageSettings(self, REQUEST,
                                        manage_tabs_message=message)
        # now set the form settings
        for key, value in result.items():
            setattr(self, key, value)
        message="Settings changed."
        
        return self.messageSettings(self, REQUEST,
                                    manage_tabs_message=message)
    
InitializeClass(EntityProperties)    
