# -*- coding: utf-8 -*-
import string
from cStringIO import StringIO
from Products.CMFCore.DirectoryView import addDirectoryViews,\
     manage_addDirectoryView
from Products.CMFCore.utils import getToolByName
from Products.Rakun import rakun_globals
from Products.Rakun.Settings import ApplicationToolMetaType,\
     DatabaseToolMetaType, ApplicationToolName, DatabaseToolName, MenuToolName,\
     MenuToolMetaType, RelationToolMetaType, RelationToolName

__version__ = "0.3"

__doc__ = """ Install script to setup Rakun in Plone Site """



type_list = ["rakun_style","rakun_javascripts","rakun_images","rakun_templates",
             "rakun_scripts"]

css_resources = ['rakun.css','grid.css','form.css','window.css','content.css',
                 'reports.css','theme.css','rakun_menu.css',
                 'jscalendar/calendar-system.css']

js_resources = ['jsolait/init.js','jsolait/lib/urllib.js','jsolait/lib/xml.js',
                'jsolait/lib/xmlrpc.js','jscalendar/calendar_stripped.js',
                'jscalendar/calendar-i18n.js','messages.js','xmlextras.js',
                'rakun.js','entity.js','form.js','grid.js','interface.js',
                'layer.js','window.js','query.js','JSCookMenu.js','theme.js',
                'jscalculator/calculator-lib.js']

def install(self):
  """install method for cmfquickinstaller tool"""
  out = StringIO()  
  typestool = getToolByName(self, 'portal_types')
  skinstool = getToolByName(self, 'portal_skins')
  csstool = getToolByName(self, 'portal_css')
  jstool = getToolByName(self, 'portal_javascripts')
  # setup rakun tools  
  addRakunTool = self.manage_addProduct['Rakun'].manage_addTool
  addRakunTool(DatabaseToolMetaType)   
  if not hasattr(self,ApplicationToolName):
    addRakunTool(ApplicationToolMetaType)    
  if not hasattr(self, MenuToolName):
    addRakunTool(MenuToolMetaType)
    
  if not hasattr(self, RelationToolName):
    addRakunTool(RelationToolMetaType)  
  # Adding Directory Views
  for type in type_list:
    if type not in skinstool.objectIds():     
      manage_addDirectoryView(skinstool,'Rakun/skins/%s'%type)
      out.write('Added "%s" directory view to portal_skins\n' %type)
    else:
      out.write('Directory view "%s" already added to portal_skins\n' %type)
  # Setup the skins
  skins = skinstool.getSkinSelections()
  for skin in skins:
    path = skinstool.getSkinPath(skin)
    path = map(string.strip, string.split(path,','))
    for type in type_list:
      if type not in path:
        try:
          path.insert(1,type)
        except ValueError:
          path.append(type)
        out.write('Added "%s" to "%s" skin\n' %(type, skin))
      else:
        out.write('Skipping "%s" skin, "%s" is already set up\n' %(skin, type))
    path = string.join(path,",")
    skinstool.addSkinSelection(skin, path)
  # javascript register
  for js in js_resources:
    if js.startswith('jsolait'):
      jstool.registerScript(js,cookable=False) # no merge
    else:      
      jstool.registerScript(js)
    out.write('registered %s\n'%js)
  # css register
  csstool.registerStylesheet('rakun_print.css',media='print')
  for css in css_resources:
    csstool.registerStylesheet(css,media='screen')
    out.write('registered %s\n'%css)
  return out.getvalue()


def uninstall(self):
  
  out = StringIO()
  typestool = getToolByName(self, 'portal_types')
  skinstool = getToolByName(self, 'portal_skins')  
  # Setup the skins
  skins = skinstool.getSkinSelections()
  for skin in skins:
    path = skinstool.getSkinPath(skin)
    path = map(string.strip, string.split(path,','))
    for type in type_list:
      if type in path:
        try:
          path.remove(type)
          out.write('Deleted "%s" to "%s" skin\n' % (type,skin))
        except:
          pass
    path = string.join(path,',')
    skinstool.addSkinSelection(skin, path)  
  return out.getvalue()










