from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Permissions import RakunView

prefix = '__rakun__'

class RequestForm:
    
    def __init__(self):
        self.form = {}
    
    def __setitem__(self, key, value):
        self.form[key] = value

    def __getitem__(self, name):
        return self.form.get(name)
    
    def get(self, name, default=None):
        return self.form.get(name, default)
    
    def set(self, key, value):
        self.form[key] = value
    
    def keys(self):
        return self.form.keys()
    
    def items(self):
        return self.form.items()
    
   
class RequestManager:
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(RakunView)
    
    def argument_get(self, key, default=None):
        arguments = self.get_arguments()
        return arguments.get(key, default)
    
    def argument_set(self, key, value):
        arguments = self.get_arguments()
        arguments[key] = value
        self.set_arguments(arguments)
    
    def get_arguments(self):
        request = self.get_request()
        return request.get('arguments', RequestForm())
    
    def set_arguments(self,arguments=RequestForm()):
        request = self.get_request()
        request['arguments'] = arguments
        self.set_request(request)
        
    def set_variable(self,key,value):
        request = self.get_request()
        request[key] = value
        self.set_request(request)
        
    def get_variable(self,key,default=None):
        request = self.get_request()
        return request.get(key,default)
        
    def get_condition(self):
        return self.get_variable('condition')
        
    def set_condition(self,condition):
        self.set_variable('condition',condition)
        
    def get_message(self):
        return self.get_variable('message')
        
    def set_message(self,message):
        self.set_variable('message',message)
        
    def get_request(self):
        request = self.REQUEST.get(prefix,{})        
        return request.get(self.getId(), {})        
    
    def set_request(self,requestobj):
        request = self.REQUEST.get(prefix,{})
        request[self.getId()] = requestobj
        self.REQUEST.set(prefix,request)    
        
    def clear_request(self):
        request = {}
        request['arguments'] = RequestForm()
        request['condition'] = None
        self.set_request(request)
       
    def _setContext(self, **args):
        for key,value in args.items():
            self.argument_set(key,value)
            
    def _clearContext(self):
        self.clear_request()

InitializeClass(RequestManager)
