from Products.Rakun.Fields import BaseField
from Products.Rakun.IObjects import ISql, IRpc
from Products.Rakun.Condition import Eq
from Products.Rakun.Settings import registerRakunClass

    
class Function:
    """
    Base SQL-99 Functions
    """
    __implements__ = (ISql, IRpc)    
    
    _name = ''
    
    def __init__(self, field):
        self.field = field          
    
    def __eq__(self, other):
        return isinstance(other, Function) and self._name == other._name and self.field == other.field
    
    def __div__(self,other):
        return Div(self,other)
    
    def __add__(self,other):
        return Add(self,other)
    
    #--------- IRpc implementation ---------------------------------------------
    def dump(self, marshaller, write):
        properties = {'__class__': self._name,'args':[self.field]}
        marshaller.dump_struct(properties, write)
    
    
    #--------- ISql implementation ---------------------------------------------    
    def render_sql_value(self, dbobject, value):
        return dbobject.setEscapeChars(value)
    
    def render_sql_insert_field(self):
        raise "Not implemented Error"
    
    def render_sql_update_field(self):
        raise "Not implemented Error"
    
    def render_sql_field(self):
        return '%s(%s)' %(self._name, self.field.render_sql_function()) # render_sql_field
    
    def render_sql_function(self):
        return self.render_sql_field()
    
    def get_sql_field_name(self):
        return self.render_sql_field()
  
    def get_sql_field_alias(self):
        return self.render_sql_field()

    def get_db_type(self):
        return 'integer'
    
    def render_sql_table(self):
        return self.field.render_sql_table()
    
    def get_sql_join_condition(self):
        return self.field.get_sql_join_condition()
    
    def render_sql_condition(self):
        raise self.render_sql_field()
    
    def render_sql_groupby(self):
        raise "Not implemented Error"        

    def render_sql_orderby(self):
        raise "Not implemented Error"

class Max(Function):

   _name = 'Max'  
   
   def get_db_type(self):
        return self.field.get_db_type()

class Count(Function):
    
    _name = 'Count'                 

class Sum(Function):
    
    _name = 'Sum'

class Average(Function):
    
    _name = 'Average'

class As(Function):

    _name = 'As'
    
    def __init__(self, field, as):
        self.field = field
        self.as = as        
    
    
    def render_sql_field(self):
        return '%s as %s' %(self.field.render_sql_field(), self.as)
    
    def get_sql_field_name(self):
        return self.as
  
    def get_sql_field_alias(self):
        return self.as
    
    def get_db_type(self):
        return self.field.get_db_type()

    def render_sql_condition(self):
        return self.as
    
    def render_sql_orderby(self):
        return self.as
        
class ASC(Function):
    
    _name = 'ASC'
           
    def render_sql_orderby(self):
        return "%s ASC" %self.field.render_sql_orderby()
    
class DESC(Function):
    
    _name = 'DESC'
        
    def render_sql_orderby(self):
        return "%s DESC" %self.field.render_sql_orderby()    
    
#-------------------------------------------------------------------------------    
class Operator(Function):
    operator = ''
    def __init__(self,field1,field2,as=''):
        self.field1 = field1
        self.field2 = field2
        self.as = as
        
    def render(self):
        result = '(%s %s %s)' %(render(self.field1),self.operator,render(self.field2))
        if self.as:
            result += 'as %s' %self.as
        return result
        
        
class Div(Operator):
    operator = '/'

class Add(Operator):
    operator = '+'

#-------------------------------------------------------------------------------    

class Relation:
    """ Relation
    """
    
    def __init__(self,foreign,field):
        self.foreign = foreign
        self.field = field
        
    # -------ISql Implementation-------------------------------------------------#
    def render_sql_value(self, dbobject, value):
        return self.field.render_sql_value(dbobject, value)
    
    def render_sql_insert_field(self, value):
        raise "Syntax Error"
       
    def render_sql_update_field(self):
        raise "Syntax Error"
      
    def render_sql_field(self):
        return "%s.%s as %s" %(self.generateEntityName(),
                               self.field.getName(), self.get_sql_field_alias())
    
    def render_sql_function(self):
        return '%s.%s' %(self.generateEntityName(),self.field.getId())
        
    def get_sql_field_name(self):
        return self.get_sql_field_alias()
        

    def get_db_type(self):
        return self.field.get_db_type()

    def get_sql_field_alias(self):
        return "%s_%s" % (self.foreign.getId(),self.field.getId())
        
      
    def render_sql_table(self):
        entity_name = self.field.getEntityName()
        return '%s as %s' %(entity_name, self.generateEntityName())
  
    def get_sql_join_condition(self):           
        master_entity = self.field.getEntity()
        primary_key = master_entity.getPrimaryField()
        primaryField = "%s.%s" % (self.generateEntityName(),primary_key.getName())
        return Eq(As(primary_key,primaryField),self.foreign)
        
    
    def render_sql_condition(self):
        return '%s.%s' %(self.generateEntityName(), self.field.getName())
        
    
    def render_sql_groupby(self):
        return self.render_sql_condition()
    
    def render_sql_orderby(self):
        return self.render_sql_condition()
    
    # ---------------------------------------------------------------------------#    
    def generateEntityName(self):
        return '%s_%s' % (self.foreign.getId(), self.field.getEntityName())