import string
from Shared.DC.ZRDB.Results import Results
from App.Extensions import getBrain
from Shared.DC.ZRDB import RDB
from cStringIO import StringIO
from Products.Rakun.Fields import BaseField
from ISql99 import ISql99
from Condition import Condition
import re

from Functions import Function, As, Count, Relation

sqlClasses = {}

def registerSqlClass(key,sqlclass):
  sqlClasses[key] = sqlclass
        
        
class Sql99:
  "Base Sql99 Class"
  
  __implements__ = (ISql99,)
  
  special_chars = ("'",)
  
  like_keyword = 'like'
  
  
  def __init__(self,connection):
    self.connection = connection
    
  def setEscapeChars(self,text):
    if isinstance(text,basestring):
      for c in self.special_chars:
        iter=re.finditer("[^\\\\]'",text)
        for i in iter:
          index = i.start()+1
          text = text[:index] + '\%s'%c + text[index+1:]
    return text

  def getResultName(self, fieldid):
    return fieldid
  
  def execQuery(self,query):
    db = self.connection()
    # max_rows parameter mysql adapter required
    result = db.query(query,max_rows=0) 
    brain=getBrain('', '')
    zc=''
    p=self.connection.aq_parent
    if type(result) is type(''):
      f=StringIO()
      f.write(result)
      f.seek(0)
      return RDB.File(f,brain,p, zc)
    else:
      return Results(result, brain, p, zc)

  def select(self,entity,fields,condition=None,orderby=[],groupby=[],offset=0,limit=0):   
    
    real_names = {}  
    def renderFieldNames(fields):
      field_names = []
      for field in fields:
        field_names.append(field.render_sql_field())
        result_name = self.getResultName(field.get_sql_field_name())
        real_names[result_name] = field.get_sql_field_alias()                  
      return ','.join(field_names)
    
    def renderJoinCondition(fields,dbobject):
      join_condition = []
      for field in fields:
        jcondition = field.get_sql_join_condition()
        
        if jcondition:
          jcondition = 'left join %s on %s' %(field.render_sql_table(),jcondition.render(dbobject))          
          if jcondition not in join_condition:
            join_condition.append(jcondition)          
          
      return ' '.join(join_condition)       
    
    table_names = '%s %s' %(entity.getEntityName(), renderJoinCondition(fields,dbobject=self))
    
    query  = "SELECT %s FROM %s " %(renderFieldNames(fields), table_names)
      
    
    if condition:      
      query += ' WHERE %s' %condition.render(dbobject=self)
      
    if groupby:      
      groupby = map(lambda field:field.render_sql_groupby(), groupby)
      query += " GROUP BY " + ",".join(groupby)
      
    if orderby:
      orderby = map(lambda field:field.render_sql_orderby(), orderby)
      query += " ORDER BY " + ",".join(orderby)
      
    if limit:
      query += " LIMIT %s OFFSET %s" %(limit, offset)
      
    result = self.execQuery(query) 
    result.real_names = real_names
    return result.dictionaries()      
    
  def rowCount(self, entity, condition=None, groupby=[]):        
    """
       rowcount of query result
    """
    result = self.select(entity, [As(Count(entity.getPrimaryField()),'row_count')], condition, groupby=groupby)
    rowCount = 0
    for r in result:
      rowCount += r['row_count']
    return rowCount
  
  def getAutoIncrementValue(self,tableid,fieldid):
    return self.getLastIncrementValue(tableid, fieldid) + 1
  
  def getLastIncrementValue(self,tableid,fieldid):
    query = "select max(%s) as currval from %s " % (fieldid, tableid)
    rec = self.execQuery(query)
    if rec:
      return int(rec[0].currval)
    else:
      return 0  
  
  # For rakun application object and database versioning
  def getVersion(self):
    query  = "SELECT version_no FROM rakun_version"
    try:
      rec = self.execQuery(query)
      if rec:
        return rec[0].version_no
    except:
      pass
    return -1
  
  def setVersion(self,version_no):
    query  = "UPDATE rakun_version set version_no=%s" %version_no
    self.execQuery(query)   
    
  #-----------------------------------------------------------------------------
    
  def insert(self,tableid,fields):
    fieldnames = []
    fieldvalues = []
    for field,value in fields:
      render_field_text = field.render_sql_insert_field()
      render_value_text = field.render_sql_value(self, value)
      fieldnames.append(render_field_text)
      fieldvalues.append(render_value_text)
      
    query = "INSERT INTO %s (%s) VALUES (%s)" % (tableid,
                                                 ','.join(fieldnames),
                                                 ','.join(fieldvalues))
    return self.execQuery(query)                                                  
    
  def update(self,tableid,fields,condition=None):
    fieldvalues = []    
    for field,value in fields:  
      render_field_text = field.render_sql_insert_field()
      render_value_text = field.render_sql_value(self, value)
      fieldvalues.append('%s=%s'%(render_field_text,
                                  render_value_text))    
    if condition:      
      condition = " WHERE %s " %condition.render(dbobject=self)
    else:
      condition = ""      
      
    query = "UPDATE %s set %s %s" %(tableid,','.join(fieldvalues),condition)
    return self.execQuery(query)
    
  def delete(self,tableid,condition):
    """
      record delete with condition
    """
    query = "DELETE FROM %s" %tableid
    if condition:      
      query += " WHERE %s" %condition.render(dbobject=self)      
    return self.execQuery(query)

  #---------------------------------------------------------------------------
  # render methods for datatypes
  #---------------------------------------------------------------------------
  def string_render(self,value):
    """  """
    if value is not None:
      return "'%s'" %value
    else:
      return 'null'
  
  def text_render(self,value):
    """  """
    return self.string_render(value)
  
  def date_render(self,value):
    """  """
    return self.string_render(value)

  def boolean_render(self,value):
    if value in [1,'1',True,'True']:
      return 'true'
    else:
      return 'false'

  def integer_render(self,value):
    try:
      value = int(value)          
      return '%s'%value
    except:
      return 'null'
  
  def float_render(self,value):
    try:
      value = float(value)          
      return '%s'%value
    except:
      return 'null'    
  
  money_render = float_render
  
  def dummy_render(self,value):
    return '%s'%value